==> openssl-cell-engine-0.1.2.tar.gz: openssl-cell-engine-0.1.2/mars_task.c <==
/*
 * Copyright 2008 Sony Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the names of the copyright holders nor the names of their
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "e_cell.h"
#include "e_cell_err.h"

#include <mars/mars.h>
#include <string.h>

struct spe_task_st
	{
	struct mars_task_id task_id;
	};

struct mars_context *mars_ctx = NULL;

static int mars_init_p = 0;
static int mars_external_ctx_p = 0;

int spe_task_set_runtime_system(void *ptr)
	{
	int rc;

	DPRINTF("\n");

	CRYPTO_w_lock(CRYPTO_LOCK_ENGINE);

	if (mars_init_p)
		{
		CELLerr(CELL_F_SPE_TASK_SET_RUNTIME_SYSTEM,
			CELL_R_RUNTIME_SYSTEM_IN_USE);
		rc = 0;
		}
	else
		{
		ASSERT(mars_external_ctx_p || !mars_ctx);

		mars_ctx = (struct mars_context *)ptr;
		mars_external_ctx_p = (mars_ctx != NULL);
		rc = 1;
		}

	CRYPTO_w_unlock(CRYPTO_LOCK_ENGINE);

	return rc;
	}

spe_task_t *spe_task_create(spe_program_handle_t *prog)
	{
	spe_task_t *task;
	int ret;
	struct mars_task_params params;

	DPRINTF("\n");

	task = aligned_malloc(MARS_TASK_ID_ALIGN, sizeof(*task));
	if (!task)
		{
		CELLerr(CELL_F_SPE_TASK_RUN, CELL_R_MEMORY_ALLOCATION_FAILED);
		return NULL;
		}

	memset(task, 0, sizeof(*task));

	memset(&params, 0, sizeof(params));
	params.name = "OpenSSL Task";
	params.elf_image = prog->elf_image;
	params.context_save_size = MARS_TASK_CONTEXT_SAVE_SIZE_MAX;

	ASSERT(mars_init_p);
	ret = mars_task_initialize(mars_ctx, &task->task_id, &params);
	if (ret != MARS_SUCCESS)
		{
		CELLerr(CELL_F_SPE_TASK_RUN, CELL_R_TASK_INITIALIZATION_FAILED);
		aligned_free(task);
		return NULL;
		}

	return task;
	}

int spe_task_destroy(spe_task_t *task)
	{
	DPRINTF("\n");

	ASSERT(mars_init_p);

	mars_task_finalize(&task->task_id);
	aligned_free(task);

	return 1;
	}

int spe_task_run(spe_task_t *task, void *arg1, void *arg2)
	{
	int ret;
	struct mars_task_args args;
	uint8_t priority;

	DPRINTF("\n");

	CRYPTO_r_lock(CRYPTO_LOCK_ENGINE);
	priority = cell_task_priority < 0 ? 0 : cell_task_priority;
	CRYPTO_r_unlock(CRYPTO_LOCK_ENGINE);

	args.type.u64[0] = (uintptr_t)arg1;
	args.type.u64[1] = (uintptr_t)arg2;

	ASSERT(mars_init_p);
	ret = mars_task_schedule(&task->task_id, &args, priority);
	if (ret != MARS_SUCCESS)
		{
		CELLerr(CELL_F_SPE_TASK_RUN, CELL_R_TASK_SCHEDULE_FAILED);
		return 0;
		}

	return 1;
	}

int spe_task_wait(spe_task_t *task)
	{
	DPRINTF("\n");

	ASSERT(mars_init_p);

	mars_task_wait(&task->task_id);

	return 1;
	}

/* spe_task_schedular_initialize (and its caller cell_init) is called
 * with CRYPTO_LOCK_ENGINE, so should be thread safe implicitly.
 */
int spe_task_schedular_initialize(void)
	{
	int ret;

	DPRINTF("\n");

	ASSERT(!mars_init_p);

	if (mars_external_ctx_p)
		{
		ASSERT(mars_ctx);
		}
	else
		{
		struct mars_params params;

		memset(&params, 0, sizeof(params));
		params.num_mpus = cell_max_spes;

		DPRINTF("%d SPEs are used\n", params.num_mpus);

		mars_ctx = OPENSSL_malloc(sizeof(*mars_ctx));
		if (!mars_ctx)
			{
			CELLerr(CELL_F_SPE_TASK_SCHEDULAR_INITIALIZE,
				CELL_R_MEMORY_ALLOCATION_FAILED);
			return 0;
			}

		ret = mars_initialize(mars_ctx, &params);
		if (ret != MARS_SUCCESS)
			{
			OPENSSL_free(mars_ctx);
			CELLerr(CELL_F_SPE_TASK_SCHEDULAR_INITIALIZE,
				CELL_R_RUNTIME_SYSTEM_INITIALIZATION_FAILED);
			return 0;
			}
		}

	mars_init_p = 1;

	return 1;
	}

/* spe_task_schedular_finalize (and its caller cell_finish) is called
 * with CRYPTO_LOCK_ENGINE, so should be thread safe implicitly.
 */
int spe_task_schedular_finalize(void)
	{
	DPRINTF("\n");

	ASSERT(mars_init_p);

	if (!mars_external_ctx_p)
		{
		mars_finalize(mars_ctx);
		OPENSSL_free(mars_ctx);
		mars_ctx = NULL;
		}

	mars_init_p = 0;

	return 1;
	}

==> openssl-cell-engine-1.0.0.tar.gz: openssl-cell-engine-1.0.0/mars_task.c <==
/*
 * Copyright 2008 Sony Corporation
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *   * Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *   * Neither the names of the copyright holders nor the names of their
 *     contributors may be used to endorse or promote products derived from this
 *     software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "e_cell.h"
#include "e_cell_err.h"

#include "mars_runtime.h"

#include <string.h>

spe_task_t *spe_task_create(spe_program_handle_t *prog)
	{
	spe_task_t *task;
	int ret;

	DPRINTF("\n");

	task = aligned_malloc(MARS_TASK_ID_ALIGN, sizeof(*task));
	if (!task)
		{
		CELLerr(CELL_F_SPE_TASK_RUN, CELL_R_MEMORY_ALLOCATION_FAILED);
		return NULL;
		}

	memset(task, 0, sizeof(*task));

	ret = mars_context_create(&task->mars, 0, 1 /* shared */);
	if (ret)
		{
		CELLerr(CELL_F_SPE_TASK_RUN, CELL_R_RUNTIME_SYSTEM_INITIALIZATION_FAILED);
		return NULL;
		}

	ret = mars_task_create(task->mars, &task->task_id,
		"OpenSSL Task", prog->elf_image, MARS_TASK_CONTEXT_SAVE_ALL);
	if (ret != MARS_SUCCESS)
		{
		CELLerr(CELL_F_SPE_TASK_RUN, CELL_R_TASK_INITIALIZATION_FAILED);
		mars_context_destroy(task->mars);
		aligned_free(task);
		return NULL;
		}

	return task;
	}

int spe_task_destroy(spe_task_t *task)
	{
	DPRINTF("\n");

	mars_task_destroy(&task->task_id);
	mars_context_destroy(task->mars);
	aligned_free(task);

	return 1;
	}

int spe_task_run(spe_task_t *task, cell_queue_t *in_q, cell_queue_t *out_q)
	{
	int ret;
	struct mars_task_args args;
	uint8_t priority;

	DPRINTF("\n");

	CRYPTO_r_lock(CRYPTO_LOCK_ENGINE);
	priority = cell_task_priority < 0 ? 0 : cell_task_priority;
	CRYPTO_r_unlock(CRYPTO_LOCK_ENGINE);

	args.type.u64[0] = (uintptr_t)in_q;
	args.type.u64[1] = (uintptr_t)out_q;

	ret = mars_task_schedule(&task->task_id, &args, priority);
	if (ret != MARS_SUCCESS)
		{
		CELLerr(CELL_F_SPE_TASK_RUN, CELL_R_TASK_SCHEDULE_FAILED);
		return 0;
		}

	return 1;
	}

int spe_task_wait(spe_task_t *task)
	{
	DPRINTF("\n");

	mars_task_wait(&task->task_id, NULL);

	return 1;
	}
